import java.applet.*;
import java.awt.*;
import java.util.*;
import CostDriverPanel;
import CostDriver;

// This class is an CostDriverPanel that displays each of the 
// driver labels and a Choice field for each
public class ChoicePanel extends CostDriverPanel
{
	Choice choiceArray[];		
	Vector costDrivers;			// holds items of type CostDriver

	// the constructor is passed the drivers that are to be
	// editable on this panel
    public ChoicePanel(Vector drivers)
    {
        super();		// construct the parent class

		// clone the drivers
		costDrivers = (Vector) drivers.clone();	

		// allocate an array of Choice fields, each cost driver
		// will be entered in a Choice field
		choiceArray = new Choice[costDrivers.size()];

		// create and use a GridBagLayout layout manager
		GridBagLayout layout = new GridBagLayout(); 
        GridBagConstraints gbc = new GridBagConstraints(); 
        setLayout(layout); 

		// use an enumeration to move through the cost drivers to
		// be displayed on this panel
		int i = 0;
		for (Enumeration e=costDrivers.elements();
				e.hasMoreElements(); i++) 
		{
			CostDriver cd = (CostDriver)e.nextElement();

			// allocate a Choice for each cost driver
			choiceArray[i] = new Choice();

			// use an enumeration to move through the ratings
			// within each cost driver (e.g., Low, Nominal, High)
			for (Enumeration enum = cd.ratings.elements(); 
					enum.hasMoreElements() ;) 
			{
				Rating r = (Rating)enum.nextElement();
				// add the name of the rating to the Choice
				choiceArray[i].addItem(r.name);
			}

			// select this item by default
			choiceArray[i].select("Nominal");		

		    // add the label to the panel
			gbc.gridwidth = 1;
			Label label = new Label(cd.name);
			gbc.anchor = GridBagConstraints.EAST;
			layout.setConstraints(label, gbc); 
			add(label);

			// add the Choice field to the panel
			gbc.gridwidth = GridBagConstraints.REMAINDER;
			gbc.anchor = GridBagConstraints.WEST;
			gbc.insets.left=5;
			layout.setConstraints(choiceArray[i], gbc); 
			add(choiceArray[i]);
		}
    }

	// Return a multiplier based on the values selected for each
	// cost driver on this panel.
	public float getMultiplier()
	{
		float value = 1.0f;
		int i = 0;

		// Use an enumeration to move through each of the cost 
		// drivers on this panel.
		for (Enumeration e=costDrivers.elements(); 
				e.hasMoreElements(); i++) 
		{
			// For each cost driver, get the rating associated
			// with the current selection in the Choice field.
			CostDriver cd = (CostDriver)e.nextElement();
			int index = choiceArray[i].getSelectedIndex();
			Rating r = (Rating)cd.ratings.elementAt(index);

			// multiply each of the multipliers together to
			// get the final result.
			value *= r.multiplier;
		}
		return value;
	}

	// Set each of the cost drivers to the Nominal value.
	public void SetDefaults()
	{
		int qty = costDrivers.size();

		for (int i=0; i < qty; i++) 
			choiceArray[i].select("Nominal");
	}

	// Create a String that summarizes the selections made on
	// this panel.
	public String GetSummary()
	{
		StringBuffer buf = new StringBuffer();

		// use an enumeration to move through each of the drivers
		int i=0;
		for (Enumeration e=costDrivers.elements(); 
				e.hasMoreElements(); i++) 
		{
			// append the name of the driver
			CostDriver cd = (CostDriver)e.nextElement();
			buf.append(cd.name+"\r\n");

			// append the name of the selected value
			buf.append("\t" + choiceArray[i].getSelectedItem() +
					"\r\n");

			// retrieve and then append the multiplier for the
			// selected value for this cost driver
			int index = choiceArray[i].getSelectedIndex();
			Rating r = (Rating)cd.ratings.elementAt(index);

			buf.append("\t" + r.multiplier + "\r\n");
		}
		return buf.toString();
	}
}



