import java.applet.*;
import java.awt.*;
import java.util.*;
import java.lang.Math;
import CostDriverPanel;
import ChoicePanel;
import ProjectPanel;
import Rating;
import SummaryFrame;

public class CostEstimator extends Applet
{
	Panel cardPanel;
	CardLayout cardLayout;
	Vector CostDriverPanelVector;
	TextField klocField;
	TextField personMonthsField;
	TextField effortAdjustmentField;
	TextField calendarMonthsField;
	ImageCanvas imageCanvas;

    public void init()
    {
		// Create a GridBagLayout and use it as the applet's
		// layout manager
        GridBagLayout layout = new GridBagLayout(); 
        setLayout(layout); 

		// create each of the main areas of the user interface
 		CreateImagemap(layout);
		CreateCardPanel(layout);
		CreateResultsPanel(layout);
		CreateButtonPanel(layout);

		resize(400, 320);
    }

	// create an imagemap that is used to select between
	// the various CostDriverPanels (Product, Computer,
	// Personnel, and Project)
	private void CreateImagemap(GridBagLayout parentLayout)
	{
		// create the image
		Image image = getImage(getDocumentBase(), "map.gif");

		// create a canvas on which the image will be displayed
		imageCanvas = new ImageCanvas(image);

		// place the canvas on the applet using the applet's
		// layout manager
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridwidth = GridBagConstraints.RELATIVE;
        gbc.fill = GridBagConstraints.BOTH;
        parentLayout.setConstraints(imageCanvas, gbc); 
        add(imageCanvas);
	}

	// create a card panel. The contents of the card panel will
	// change depending on what has been selected in the imagemap.
	private void CreateCardPanel(GridBagLayout parentLayout)
	{
		// create the panel
		cardPanel = new Panel();

		// place the panel on the applet using the applet's
		// layout manager
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx=1;
        parentLayout.setConstraints(cardPanel, gbc); 
		add(cardPanel);

		// create a CardLayout and assign it to the panel
		cardLayout = new CardLayout();
		cardPanel.setLayout(cardLayout);

		// create a vector to hold the CostDriverPanels that 
		// will be placed on cardPanel
		CostDriverPanelVector = new Vector(4);

		// create each of the panels that can be displayed 
		// on this card
		CreateProductPanel();
		CreateComputerPanel();
		CreatePersonnelPanel();
		CreateProjectPanel();
	}

	// create the product panel that can be displayed on cardPanel
	// on the applet.
	private void CreateProductPanel()
	{
		// allocate a vector that will hold the cost drivers 
		// that are displayed on this panel
		Vector costDrivers = new Vector(3);

		// create a vector of Reliability ratings
		Vector relyRatings = new Vector(5);
		relyRatings.addElement(new Rating("Very low",   0.75F));
		relyRatings.addElement(new Rating("Low",        0.88F));
		relyRatings.addElement(new Rating("Nominal",    1.00F));
		relyRatings.addElement(new Rating("High",       1.15F));
		relyRatings.addElement(new Rating("Very high",  1.40F));
		// create the Reliability CostDriver and add it to the 
		// vector
		costDrivers.addElement(new CostDriver("Reliability", 
				relyRatings));

		// create a vector of Database Size ratings
		Vector dataRatings = new Vector(4);
		dataRatings.addElement(new Rating("Low",        0.94F));
		dataRatings.addElement(new Rating("Nominal",    1.00F));
		dataRatings.addElement(new Rating("High",       1.08F));
		dataRatings.addElement(new Rating("Very high",  1.16F));
		// create the Database Size CostDriver and add it to the 
		// vector
		costDrivers.addElement(new CostDriver("Database Size", 
				dataRatings));

		// create a vector of Complexity ratings
		Vector cplxRatings = new Vector(6);
		cplxRatings.addElement(new Rating("Very low",   0.70F));
		cplxRatings.addElement(new Rating("Low",        0.85F));
		cplxRatings.addElement(new Rating("Nominal",    1.00F));
		cplxRatings.addElement(new Rating("High",       1.15F));
		cplxRatings.addElement(new Rating("Very high",  1.30F));
		cplxRatings.addElement(new Rating("Extra high", 1.65F));
		// create the Complexity CostDriver and add it to the 
		// vector
		costDrivers.addElement(new CostDriver("Complexity", 
				cplxRatings));

		// create the new panel and add it to the card
		ChoicePanel productPanel = new ChoicePanel(costDrivers);
		cardPanel.add("Product", productPanel);
		
		// add this panel to the vector of all CostDriverPanels
		CostDriverPanelVector.addElement(productPanel);
	}

	// create the computer panel that can be displayed on cardPanel
	// on the applet.
	private void CreateComputerPanel()
	{
		// allocate a vector that will hold the cost drivers 
		// that are displayed on this panel
		Vector costDrivers = new Vector(4);

		// create a vector of Execution Time ratings
		Vector execRatings = new Vector(4);
		execRatings.addElement(new Rating("Nominal",    1.00F));
		execRatings.addElement(new Rating("High",       1.11F));
		execRatings.addElement(new Rating("Very high",  1.30F));
		execRatings.addElement(new Rating("Extra high", 1.66F));
		// create the Execution Time CostDriver and add it to the 
		// vector
		costDrivers.addElement(new CostDriver("Execution Time", 
				execRatings));

		// create a vector of Storage Constraint ratings
		Vector storRatings = new Vector(4);
		storRatings.addElement(new Rating("Nominal",    1.00F));
		storRatings.addElement(new Rating("High",       1.06F));
		storRatings.addElement(new Rating("Very high",  1.21F));
		storRatings.addElement(new Rating("Extra high", 1.56F));
		// create the Storage Constraint CostDriver and add it to
		// the vector
		costDrivers.addElement(new CostDriver("Storage Constraint",
				storRatings));

		// create a vector of Virtual Machine Volatility ratings
		Vector virtRatings = new Vector(4);
		virtRatings.addElement(new Rating("Low",        0.87F));
		virtRatings.addElement(new Rating("Nominal",    1.00F));
		virtRatings.addElement(new Rating("High",       1.15F));
		virtRatings.addElement(new Rating("Very high",  1.30F));
		// create the Virtual Machine Volatility CostDriver and 
		// add it to the vector
		costDrivers.addElement(new CostDriver(
				"Virtual Machine Volatility", virtRatings));

		// create a vector of Turnaround Time ratings
		Vector turnRatings = new Vector(4);
		turnRatings.addElement(new Rating("Low",        0.87F));
		turnRatings.addElement(new Rating("Nominal",    1.00F));
		turnRatings.addElement(new Rating("High",       1.07F));
		turnRatings.addElement(new Rating("Very high",  1.15F));
		// create the Turnaround Time CostDriver and add it to
		// the vector
		costDrivers.addElement(new CostDriver("Turnaround Time", 
				turnRatings));

		// create the new panel and add it to the card
		ChoicePanel computerPanel = new ChoicePanel(costDrivers);
		cardPanel.add("Computer", computerPanel);
		
		// add this panel to the vector of all CostDriverPanels
		CostDriverPanelVector.addElement(computerPanel);
	}

	// create the personnel panel that can be displayed on 
	// cardPanel on the applet.
	private void CreatePersonnelPanel()
	{
		// allocate a vector that will hold the cost drivers 
		// that are displayed on this panel
		Vector costDrivers = new Vector(5);

		// create a vector of Analyst Capability ratings
		Vector acapRatings = new Vector(5);
		acapRatings.addElement(new Rating("Very low",   1.46F));
		acapRatings.addElement(new Rating("Low",        1.19F));
		acapRatings.addElement(new Rating("Nominal",    1.00F));
		acapRatings.addElement(new Rating("High",       0.86F));
		acapRatings.addElement(new Rating("Very high",  0.71F));
		// create the Analyst Capability CostDriver and add it to
		// the vector
		costDrivers.addElement(new CostDriver("Analyst Capability",
				acapRatings));

		// create a vector of Applications Experience ratings
		Vector aexpRatings = new Vector(5);
		aexpRatings.addElement(new Rating("Very low",   1.29F));
		aexpRatings.addElement(new Rating("Low",        1.13F));
		aexpRatings.addElement(new Rating("Nominal",    1.00F));
		aexpRatings.addElement(new Rating("High",       0.91F));
		aexpRatings.addElement(new Rating("Very high",  0.82F));
		// create the Applications Experience CostDriver and add 
		// it to the vector
		costDrivers.addElement(new CostDriver(
				"Applications Experience", aexpRatings));
	
		// create a vector of Programmer Capability ratings
		Vector pcapRatings = new Vector(5);
		pcapRatings.addElement(new Rating("Very low",   1.42F));
		pcapRatings.addElement(new Rating("Low",        1.17F));
		pcapRatings.addElement(new Rating("Nominal",    1.00F));
		pcapRatings.addElement(new Rating("High",       0.86F));
		pcapRatings.addElement(new Rating("Very high",  0.70F));
		// create the Programmer Capability CostDriver and add 
		// it to the vector
		costDrivers.addElement(new CostDriver(
				"Programmer Capability", pcapRatings));

		// create a vector of Virtual Machine Experience ratings
		Vector vexpRatings = new Vector(4);
		vexpRatings.addElement(new Rating("Very low",   1.21F));
		vexpRatings.addElement(new Rating("Low",        1.10F));
		vexpRatings.addElement(new Rating("Nominal",    1.00F));
		vexpRatings.addElement(new Rating("High",       0.90F));
		// create the Virtual Machine Experience CostDriver and 
		// add it to the vector
		costDrivers.addElement(new CostDriver(
				"Virtual Machine Experience", vexpRatings));

		// create a vector of Language Experience ratings
		Vector lexpRatings = new Vector(4);
		lexpRatings.addElement(new Rating("Very low",   1.14F));
		lexpRatings.addElement(new Rating("Low",        1.07F));
		lexpRatings.addElement(new Rating("Nominal",    1.00F));
		lexpRatings.addElement(new Rating("High",       0.95F));
		// create the Language Experience CostDriver and add 
		// it to the vector
		costDrivers.addElement(new CostDriver(
				"Language Experience", lexpRatings));

		// create the new panel and add it to the card
		ChoicePanel personnelPanel = new ChoicePanel(costDrivers);
		cardPanel.add("Personnel", personnelPanel);
		
		// add this panel to the vector of all CostDriverPanels
		CostDriverPanelVector.addElement(personnelPanel);
	}

	// create the project panel that can be displayed on cardPanel
	// on the applet.
	private void CreateProjectPanel()
	{
		// create the new panel and add it to the card
		ProjectPanel projectPanel = new ProjectPanel();
		cardPanel.add("Project", projectPanel);
		
		// add this panel to the vector of all CostDriverPanels
		CostDriverPanelVector.addElement(projectPanel);
	}

	// create a panel to display the results of running the applet
	private void CreateResultsPanel(GridBagLayout parentLayout)
	{
		// create a panel and add it to the applet using the 
		// applet's layout manager
		Panel p = new Panel();
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridwidth = GridBagConstraints.RELATIVE;
        gbc.fill = GridBagConstraints.BOTH;
        parentLayout.setConstraints(p, gbc); 
        add(p);

		// create and assign a GridBagLayout as the layout manager
		// for this panel
		GridBagLayout layout = new GridBagLayout();
		p.setLayout(layout);

		// place some space at the top of the panel
		Panel space = new Panel();
		gbc.gridheight =3;
		gbc.weighty =1;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.fill = GridBagConstraints.BOTH;
        layout.setConstraints(space, gbc); 
        p.add(space);
		gbc.weighty = 0;

		// create a Label and TextField for KLOC
		Label klocLabel = new Label("KLOC:");
		gbc.gridheight = 1;
		gbc.gridwidth = GridBagConstraints.RELATIVE;
        layout.setConstraints(klocLabel, gbc); 
        p.add(klocLabel);

		gbc.gridwidth = GridBagConstraints.REMAINDER;
		klocField = new TextField(5);
        layout.setConstraints(klocField, gbc); 
        p.add(klocField);

		// create a Label and TextField for Effort Adjustment
		Label effortAdjustLabel = new Label("Effort Adjustment:");
		gbc.gridwidth = GridBagConstraints.RELATIVE;
        layout.setConstraints(effortAdjustLabel, gbc); 
        p.add(effortAdjustLabel);

		gbc.gridwidth = GridBagConstraints.REMAINDER;
		effortAdjustmentField = new TextField(5);
		// make the field read-only
		effortAdjustmentField.setEditable(false);
        layout.setConstraints(effortAdjustmentField, gbc); 
        p.add(effortAdjustmentField);

		// create a Label and TextField for Person Months
		Label personMonthsLabel = new Label("Person Months:");
		gbc.gridwidth = GridBagConstraints.RELATIVE;
        layout.setConstraints(personMonthsLabel, gbc); 
        p.add(personMonthsLabel);

		gbc.gridwidth = GridBagConstraints.REMAINDER;
		personMonthsField = new TextField(5);
		// make the field read-only
		personMonthsField.setEditable(false);
        layout.setConstraints(personMonthsField, gbc); 
        p.add(personMonthsField);

		// create a Label and TextField for Calendar Months
		Label calendarMonthsLabel = new Label("Calendar Months:");
		gbc.gridwidth = GridBagConstraints.RELATIVE;
        layout.setConstraints(calendarMonthsLabel, gbc); 
        p.add(calendarMonthsLabel);

		gbc.gridwidth = GridBagConstraints.REMAINDER;
		calendarMonthsField = new TextField(5);
		// make the field read-only
		calendarMonthsField.setEditable(false);
        layout.setConstraints(calendarMonthsField, gbc); 
        p.add(calendarMonthsField);
	
		// create some space at the bottom of the panel
		// so the fields above are properly located
		space = new Panel();
		gbc.gridheight =3;
		gbc.weighty =1;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.fill = GridBagConstraints.BOTH;
        layout.setConstraints(space, gbc); 
        p.add(space);
	}

	// create a panel to hold Summary, Recalc, and Reset buttons
	private void CreateButtonPanel(GridBagLayout parentLayout)
	{
		// create a panel and add it to the applet using the 
		// applet's layout manager
		Panel p = new Panel();
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridwidth = GridBagConstraints.REMAINDER;
        parentLayout.setConstraints(p, gbc); 
        add(p);

		// create and assign a GridBagLayout as the layout manager
		// for this panel
		GridBagLayout layout = new GridBagLayout();
		p.setLayout(layout);

		// place some space above the buttons
		Panel space = new Panel();
		gbc.gridheight =3;
		gbc.weighty =1;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.fill = GridBagConstraints.BOTH;
        layout.setConstraints(space, gbc); 
        p.add(space);

		// place some space to the left of the buttons
		space = new Panel();
		gbc.gridheight = 4;
		gbc.weighty = 1;
		gbc.gridwidth = GridBagConstraints.RELATIVE;
		gbc.weightx=1;
        layout.setConstraints(space, gbc); 
        p.add(space);
		gbc.gridheight = 1;
		gbc.gridwidth = GridBagConstraints.REMAINDER;

		// add the Summary button
        gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weightx =1;
		gbc.anchor = GridBagConstraints.EAST;
		Button b = new Button("Summary");
        layout.setConstraints(b, gbc); 
        p.add(b);

		// add the Recalc button
		b = new Button("Recalc");
        layout.setConstraints(b, gbc); 
        p.add(b);

		// add the Reset button
		b = new Button("Reset");
        layout.setConstraints(b, gbc); 
        p.add(b);
	}

	// watch for a press of the Reset, Recalc, or Summary button
	// and then call the appropriate method
    public boolean action(Event evt, Object obj)
    {
        boolean result=false;

        if("Reset".equals(obj)) 
		{
			DoReset();
            result = true;
        }

        else if("Recalc".equals(obj)) 
		{
			DoRecalc();
            result = true;
        }

        else if("Summary".equals(obj)) 
		{
			ShowSummary();
            result = true;
        }
		return result;
	}

	// reset each of the cost drivers to a default value
	private void DoReset()
	{
		// use an enumeration to move through the CostDriverPanels
		for (Enumeration enum = CostDriverPanelVector.elements();
				enum.hasMoreElements(); )
		{
			// for each panel, call SetDefaults
			CostDriverPanel p = (CostDriverPanel)enum.nextElement();
			p.SetDefaults();
		}
	}

	// recalc the schedule for the project based on the current
	// cost driver values
	private void DoRecalc()
	{
		try 
		{
			// make sure a number was entered in klocField
			Float kloc = Float.valueOf(klocField.getText());

			// for this level of semidetached COCOMO, start with 
			// 1.12 as the base effort adjustment factor
			float effortAdjustment = 1.12F;

			// calculate the Effort Adjustment Factor by calling
			// getMultiplier for each CostDriverPanel
			for (Enumeration e=CostDriverPanelVector.elements(); 
					e.hasMoreElements(); )
			{
				CostDriverPanel p = (CostDriverPanel)e.nextElement();
				effortAdjustment *= p.getMultiplier();
			}
			// display the Effort Adjustment Factor
			effortAdjustmentField.setText(String.valueOf(
					effortAdjustment));

			// calculate and display the number of person months 
			// on the project
			int personMonths=(int)(3.0D*(Math.pow(kloc.floatValue(),
					effortAdjustment)));
			personMonthsField.setText(String.valueOf(personMonths));

			// calculate and display the number of calendar months
			// on the project
			double calendarMonths=2.5*(Math.pow(personMonths, 0.35F));
			calendarMonthsField.setText(String.valueOf(Math.round(
					calendarMonths)));
		}
		catch (NumberFormatException e)
		{
			// if klocField doesn't contain a number, set all 
			// the fields to blank
			effortAdjustmentField.setText("");
			personMonthsField.setText("");
			calendarMonthsField.setText("");
		}
	}

	// display summary information about the cost drivers
	private void ShowSummary()
	{
		SummaryFrame frame=new SummaryFrame(CostDriverPanelVector);
        frame.show();
	}

	// if the mouse button is pressed, check to see if the pointer
	// was over the imagemap
	public boolean mouseDown(Event e, int x, int y)
	{
		StringBuffer cardName = new StringBuffer();

		// if the button was pressed when the mouse pointer was 
		// over the imagemap, get the name of the card to display
		// in the cardPanel.
		if (imageCanvas.NewImageSelected(x,y,cardName) == true) {
            cardLayout.show(cardPanel, cardName.toString());
			imageCanvas.repaint();
		}
		return true;
	}
}



