import java.util.*;
import java.awt.*;

// This class is used to display an image on the applet
public class ImageCanvas extends Canvas 
{
	private Image image;
	private Vector rects;
	private ImagemapRect lastrect;

	// construct an image canvas based on the specified image
	// and the currently selected portion of the imagemap
	ImageCanvas(Image image)
	{
		resize(50,50);
		this.image=image;

		// allocate a vector to hold the ImagemapRects 
		rects = new Vector();

		// all the ImagemapRects that describe the hotspots
		// on the imagemap
		rects.addElement(new ImagemapRect(0,   0, 50, 50, 
				"Product"));
		rects.addElement(new ImagemapRect(49,  0, 50, 50, 
				"Computer"));
		rects.addElement(new ImagemapRect(0,  49, 50, 50, 
				"Personnel"));
		rects.addElement(new ImagemapRect(49, 49, 50, 50, 
				"Project"));

		// store the last selected ImagemapRect
		lastrect = (ImagemapRect)rects.elementAt(0);

	}

	public void paint(Graphics g)
	{
		// paint the image
		g.drawImage(image, 0, 0, this);

		// highlight the selected location
		g.setXORMode(Color.red);
		g.drawRect(lastrect.x, lastrect.y, lastrect.width, 
				lastrect.height);
		g.drawRect(lastrect.x+1, lastrect.y+1, lastrect.width-2,
				lastrect.height-2);
		g.drawRect(lastrect.x+2, lastrect.y+2, lastrect.width-4,
				lastrect.height-4);
	}

	// determine which ImageMapRect (if any) the mouse pointer
	// is in
	private ImagemapRect InWhichRect(int x, int y)
	{
		// use an enumeration to move through each ImagemapRect
		// until finding one which encloses the point (x, y)
        for (Enumeration enum = rects.elements(); 
				enum.hasMoreElements(); )
		{
			ImagemapRect r = (ImagemapRect)enum.nextElement();
			if (r.inside(x, y))
				return r;
		}
		return null;
	}

	// Determine if the mouse pointer was clicked over an area in
	// the imagemap that isn't already the current selection
	public boolean NewImageSelected(int x, int y, 
			StringBuffer cardName)
	{
		boolean result = false;		// assume failure

		// normalize the coordinates relative to the ImageCanvas
		int xPos = x - bounds().x;
		int yPos = y - bounds().y;

		// determine which rectangle (if any) was selected
		ImagemapRect r = InWhichRect(xPos, yPos);

		// if a rectangle was selected and it isn't the same
		// one that is already selected, a new area has been
		// selected
		if (r != null && r != lastrect)
		{
			lastrect = r;
			cardName.append(r.cardName);
			result = true;
		}
		return result;
	}

}

// This class extends Rectangle by storing a name. This is the name
// of the CostDriverPanel that will be displayed when this 
// rectangle in the imagemap is selected.
class ImagemapRect extends Rectangle {
	String cardName;
	public ImagemapRect(int x, int y,int w,int h, String cardName)
	{
		super(x,y,w,h);
		this.cardName = cardName;
	}
}
