import java.applet.*;
import java.awt.*;
import java.util.*;
import CostDriverPanel;
import CostDriver;


// This class is an CostDriverPanel that uses both Choices and
// Checkboxes to get user input.
public class ProjectPanel extends CostDriverPanel
{
	Choice choiceArray[];
	Vector costDrivers;
	Vector cbVector;
	CheckboxGroup cbGroup;

    public ProjectPanel()
    {
        super();

		LoadCostDrivers();		  // load values into costDrivers

		// allocate an array of Choice fields that will hold
		// all but one cost driver
		int qty = costDrivers.size();	
		choiceArray = new Choice[qty - 1];

		// create and use a GridBagLayout layout manager
        GridBagLayout layout = new GridBagLayout(); 
        GridBagConstraints gbc = new GridBagConstraints(); 
        setLayout(layout); 

		// Loop through the cost drivers creating a Choice field
		// for each. Process all but the last one. The last one 
		// will be displayed using Checkboxes instead of a Choice
		// field.
		int i = 0;
		for (i=0; i<qty-1; i++) 
		{
			CostDriver cd = (CostDriver)costDrivers.elementAt(i);

			// allocate a Choice for each cost driver
			choiceArray[i] = new Choice();

			// use an enumeration to move through the ratings
			// within each cost driver (e.g., Low, Nominal, High)
			for (Enumeration enum = cd.ratings.elements(); 
					enum.hasMoreElements() ;) 
			{
				Rating r = (Rating)enum.nextElement();
				// add the name of the rating to the Choice
				choiceArray[i].addItem(r.name);
			}

			// select this item by default
			choiceArray[i].select("Nominal");

		    // add the label to the panel
			gbc.gridwidth = 1;
			Label label = new Label(cd.name);
			gbc.anchor = GridBagConstraints.EAST;
			layout.setConstraints(label, gbc); 
			add(label);

		    // add the Choice field to the panel
			gbc.gridwidth = GridBagConstraints.REMAINDER;
			gbc.anchor = GridBagConstraints.WEST;
			gbc.insets.left=5;
			layout.setConstraints(choiceArray[i], gbc); 
			add(choiceArray[i]);
		}

		// use the last cost driver to make a CheckboxGroup
		CostDriver cd = (CostDriver)costDrivers.elementAt(qty-1);
	
		// create a vector to hold the Checkboxes
		cbVector = new Vector(6);

		// add the label for the Checkbox group
		gbc.gridwidth = 1;
		Label label = new Label(cd.name);
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.anchor = GridBagConstraints.WEST;
		layout.setConstraints(label, gbc); 
		add(label);

		// keep track of how many items are on each line, so that
		// items can be properly positioned.
		int itemsOnLine=0;

		// create a new CheckboxGroup
		cbGroup = new CheckboxGroup();

		// all the checkbox items will be anchored on the west
		// side, so set this value before the loop
		gbc.anchor = GridBagConstraints.WEST;

		// use an enumeration to move through the ratings for the
		// cost driver
		for (Enumeration enum = cd.ratings.elements(); 
				enum.hasMoreElements(); itemsOnLine++) 
		{
			Rating r = (Rating)enum.nextElement();

			// the "Nominal" item should be checked by default
			boolean checked = r.name.equals("Nominal");

			// if line is almost full of items, tell this item to
			// use the remainder of the space
			if (itemsOnLine == 2)
			{
				gbc.gridwidth = GridBagConstraints.REMAINDER;
				itemsOnLine = 0;
			}
			else
				gbc.gridwidth = 1;

			// create the new checkbox and add it to the screen
			Checkbox cb = new Checkbox(r.name, cbGroup, checked);
			layout.setConstraints(cb, gbc); 
			add(cb);

			// then add it to the vector of checkboxes
			cbVector.addElement(cb);
		}
    }

	// create a vector of CostDrivers for this panel
	private void LoadCostDrivers()
	{
		// allocate a vector that will hold the cost drivers 
		// that are displayed on this panel
		costDrivers = new Vector(3);

		// create a vector of Modern Practices ratings
		Vector modpRatings = new Vector(5);
		modpRatings.addElement(new Rating("Very low",   1.24F));
		modpRatings.addElement(new Rating("Low",        1.10F));
		modpRatings.addElement(new Rating("Nominal",    1.00F));
		modpRatings.addElement(new Rating("High",       0.91F));
		modpRatings.addElement(new Rating("Very high",  0.82F));
		// create the Modern Practices CostDriver and add it to
		// the vector
		costDrivers.addElement(new CostDriver("Modern Practices",
				modpRatings));

		// create a vector of Software Tools ratings
		Vector toolRatings = new Vector(5);
		toolRatings.addElement(new Rating("Very low",   1.24F));
		toolRatings.addElement(new Rating("Low",        1.10F));
		toolRatings.addElement(new Rating("Nominal",    1.00F));
		toolRatings.addElement(new Rating("High",       0.91F));
		toolRatings.addElement(new Rating("Very high",  0.83F));
		// create the Software Tools CostDriver and add it to
		// the vector
		costDrivers.addElement(new CostDriver("Software Tools",
				toolRatings));

		// create a vector of Schedule Constraints ratings
		Vector scedRatings = new Vector(5);
		scedRatings.addElement(new Rating("Very low",   1.23F));
		scedRatings.addElement(new Rating("Low",        1.08F));
		scedRatings.addElement(new Rating("Nominal",    1.00F));
		scedRatings.addElement(new Rating("High",       1.04F));
		scedRatings.addElement(new Rating("Very high",  1.10F));
		// create the Schedule Constraints CostDriver and add 
		// it to the vector
		costDrivers.addElement(new CostDriver(
				"Schedule Constraints",	scedRatings));
	}

	// Return a multiplier based on the values selected for each
	// cost driver on this panel.
	public float getMultiplier()
	{
		float value = 1.0f;

		int choiceQty = costDrivers.size() - 1;

		// Loop through the items that use a Choice field
		for (int i=0; i < choiceQty; i++) 
		{
			// For each cost driver, get the rating associated
			// with the current selection in the Choice field.
			CostDriver cd = (CostDriver)costDrivers.elementAt(i);
			int index = choiceArray[i].getSelectedIndex();
			Rating r = (Rating)cd.ratings.elementAt(index);

			// multiply each of the multipliers together to
			// get the final result.
			value *= r.multiplier;
		}

		// get the CostDriver for the field that uses Checkboxes
		int lastIndex = costDrivers.size() - 1;
		CostDriver cd=(CostDriver)costDrivers.elementAt(lastIndex);

		// get the rating based on the selected checkbox
		Rating r = (Rating)cd.ratings.elementAt(whichCheckbox());

		// and multiply the rating by the current rating
		value *= r.multiplier;

		return value;
	}

	// returns an integer indicating which Checkbox is currently
	// selected
	private int whichCheckbox()
	{
		// get the current Checkbox
		Checkbox current = cbGroup.getCurrent();

		// use an enumeration to move through each Checkbox in
		// the vector
		int index=0;
		for (Enumeration e = cbVector.elements(); 
				e.hasMoreElements(); index++) 
		{
			Checkbox cb = (Checkbox)e.nextElement();

			// compare the two Checkboxes and break if equal
			if (cb.equals(current))
				break;
		}
		return index;
	}

	// Set each of the cost drivers to the Nominal value.
	public void SetDefaults()
	{
		// set the defaults for the Choice fields
		int choiceQty = costDrivers.size() - 1;
		for (int i=0; i < choiceQty; i++) 
			choiceArray[i].select("Nominal");

		// set the default Checkbox
		for (Enumeration enum = cbVector.elements(); 
				enum.hasMoreElements();) 
		{
			Checkbox cb = (Checkbox)enum.nextElement();
			if (cb.getLabel().equals("Nominal"))
			{
				cbGroup.setCurrent(cb);
				break;
			}
		}
	}

	// Create a String that summarizes the selections made on
	// this panel.
	public String GetSummary()
	{
		StringBuffer buf = new StringBuffer();

		// loop through each of the cost drivers that uses a 
		// Choice field
		int choiceQty = costDrivers.size() - 1;
		for(int i=0; i<choiceQty; i++)
		{
			// append the name of the driver
			CostDriver cd = (CostDriver)costDrivers.elementAt(i);
			buf.append(cd.name + "\r\n");

			// append the name of the selected value
			buf.append("\t" + choiceArray[i].getSelectedItem() + 
					"\r\n");

			// retrieve and then append the multiplier for the
			// selected value for this cost driver
			int index = choiceArray[i].getSelectedIndex();
			Rating r = (Rating)cd.ratings.elementAt(index);
			buf.append("\t" + r.multiplier + "\r\n");
		}

		// get info about the item that is displayed as a Checkbox
		int lastIndex = costDrivers.size() - 1;
		CostDriver cd=(CostDriver)costDrivers.elementAt(lastIndex);

		// append the name of the driver
		buf.append(cd.name + "\r\n");

		// append the label of the selected Checkbox
		Checkbox cb = cbGroup.getCurrent();
		buf.append("\t" + cb.getLabel() + "\r\n");

		// get the rating for the checked item
		Rating r = (Rating)cd.ratings.elementAt(whichCheckbox());
		buf.append("\t" + r.multiplier + "\r\n");

		return buf.toString();
	}
}
